import json
import time
import urllib.request
import urllib.error
import urllib.parse

from luksdk.exception import LukSDKExceptions
from luksdk.signature import signature


class Apis:
    """API 调用类"""

    def __init__(self, luk_sdk):
        self.luk_sdk = luk_sdk

    def get_game_service_list(self, req):
        json_str = self._make_request("/sdk/get_game_service_list", req)
        return json.loads(json_str, object_hook=lambda d: type('Response', (), d)()) 


    def query_notify_event(self, req):
        """查询通知事件"""
        json_str = self._make_request("/sdk/query_notify_event", req)
        return json.loads(json_str, object_hook=lambda d: type('Response', (), d)()) 


    def query_order(self, req):
        """查询订单"""
        json_str = self._make_request("/sdk/query_order", req)
        return json.loads(json_str, object_hook=lambda d: type('Response', (), d)()) 


    def publish_control_event(self, req):
        """发布控制事件"""
        json_str = self._make_request("/sdk/publish_control_event", req)
        return json.loads(json_str, object_hook=lambda d: type('Response', (), d)()) 

    def _prepare_request(self, req_data):
        """准备请求数据"""
        # 设置默认值
        if hasattr(req_data, 'app_id') and req_data.app_id == 0:
            req_data.app_id = self.luk_sdk.config.app_id

        # 设置时间戳
        if hasattr(req_data, 'timestamp') and req_data.timestamp is None:
            req_data.timestamp = int(time.time())

        # 生成签名
        if hasattr(req_data, 'sign') and req_data.sign is None or req_data.sign == "":
            req_data.sign = signature(self.luk_sdk.config.app_secret, req_data)
        elif isinstance(req_data, dict) and req_data.get('sign') is None or req_data.get('sign') == "":
            req_data['sign'] = signature(self.luk_sdk.config.app_secret, req_data)

        return req_data

    def _make_request(self, endpoint: str, req_data):
        """发起 HTTP 请求"""
        # 准备请求数据
        req_data = self._prepare_request(req_data)

        # 准备请求数据
        if hasattr(req_data, 'model_dump'):
            json_data = req_data.model_dump(exclude_unset=True)
        elif hasattr(req_data, 'dict'):
            json_data = req_data.dict(exclude_unset=True)
        else:
            json_data = req_data

        # 发起请求
        url = self.luk_sdk.config.domain + endpoint

        try:
            # Convert data to JSON string
            json_string = json.dumps(json_data)
            data = json_string.encode('utf-8')
            
            # Create request
            req = urllib.request.Request(url, data=data)
            req.add_header('Content-Type', 'application/json')
            
            # Make request
            response = urllib.request.urlopen(req)
            result = response.read().decode('utf-8')
            return result
        except urllib.error.HTTPError as e:
            raise LukSDKExceptions.INTERNAL_ERROR.with_detail("HTTP request failed: {0}".format(e))
        except urllib.error.URLError as e:
            raise LukSDKExceptions.INTERNAL_ERROR.with_detail("HTTP request failed: {0}".format(e))
        except ValueError as e:  # json.JSONDecodeError doesn't exist in Python 3.1
            raise LukSDKExceptions.INTERNAL_ERROR.with_detail("JSON decode failed: {0}".format(e))
