import hashlib
# typing module not available in Python 3.1

def signature(sign_secret, params):
    """生成签名"""
    params_map = _cast_to_signature_params(params)
    return _generate_signature(sign_secret, params_map)


def _generate_signature(app_secret, params):
    """生成签名"""
    # 提取并排序参数键（按 ASCII 字典序）
    keys = sorted(params.keys())

    # 构建签名字符串（过滤空值）
    signature_parts = []
    for k in keys:
        value = params[k]
        if value:  # 仅保留非空字符串（空字符串被过滤）
            signature_parts.append("{0}={1}".format(k, value))

    # 拼接签名字符串和密钥
    signature_string = "&".join(signature_parts) + "&key={0}".format(app_secret)

    # 生成 MD5 哈希并转换为大写
    hash_result = hashlib.md5(signature_string.encode('utf-8')).hexdigest().upper()

    return hash_result


def _cast_to_signature_params(obj):
    """将对象转换为签名参数（仅基本数据类型，排除 sign）"""
    result = {}

    if isinstance(obj, dict):
        # 处理字典：排除 sign 键，值转为字符串
        for key, value in obj.items():
            if value is not None and str(key) != "sign":
                result[str(key)] = str(value)
    elif hasattr(obj, 'model_dump'):
        # 处理 Pydantic v2 模型：排除未设置字段、sign 属性，非基本类型过滤
        model_dict = obj.model_dump(exclude_unset=True)
        for attr, value in model_dict.items():
            if value is not None and attr != "sign":
                if not isinstance(value, (dict, list, tuple, set)) and not callable(value):
                    result[attr] = str(value)
    elif hasattr(obj, 'dict'):
        # 处理 Pydantic v1 模型：排除未设置字段、sign 属性，非基本类型过滤
        model_dict = obj.dict(exclude_unset=True)
        for attr, value in model_dict.items():
            if value is not None and attr != "sign":
                if not isinstance(value, (dict, list, tuple, set)) and not callable(value):
                    result[attr] = str(value)
    else:
        # 处理普通对象：排除私有属性、sign 属性，非基本类型过滤
        for attr in dir(obj):
            if not attr.startswith('_') and attr != "sign":
                value = getattr(obj, attr)
                if value is not None and not isinstance(value, (dict, list, tuple, set)) and not callable(value):
                    result[attr] = str(value)

    return result